//	GamePlayState.cpp - The main gameplay state.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "CaterpillarGlobal.h"
#include "PastarenaSounder.h"
#include "drawing/Drawer.h"
#include "GamePlayState.h"
#include "ColourHelpers.h"
#include "Random.h"

//------------------------------------------------------------------------------
GameStateCreator<GamePlayState> GamePlayState::creator;

//------------------------------------------------------------------------------
GamePlayState::GamePlayState(Application& app, SounderPtr& sound):
GameState(app, sound),
backgroundTween(background, background, 0.1f),
playerPerc(-1),
pickupCount(180.0f),
pickupLeftTween(pickupLeftCol, pickupLeftCol, 0.01f),
pickupRightTween(pickupRightCol, pickupRightCol, 0.01f),
ghostLoop1(false),
ghostLoop2(false),
ghostLoop1Count(0),
ghostLoop2Count(0),
noLoops(false),
bulletCounter(0.0f),
fireIsDown(false),
directionKey(-1),
fillPos(0, 0),
musicPlaying(true),
distortion(false),
sounder(dynamic_pointer_cast<PastarenaSounder>(sound))
{
	ThreeFloats tempCol;

	globalData = dynamic_pointer_cast<CaterpillarGlobal>(getGlobalData());

	background.x = Random::getGlobal().getRandomFloat();
	background.y = 0.66f;
	background.z = 0.75f;
	background = ColourHelpers::hslToRgb(background);
	backgroundTween.reset(background);
	backgroundTween.setIndex(1.0f);

	pickupLeftCol.x = Random::getGlobal().getRandomFloat();
	pickupLeftCol.y = 0.66f;
	pickupLeftCol.z = 0.75f;
	pickupLeftCol = ColourHelpers::hslToRgb(pickupLeftCol);

	pickupRightCol.x = Random::getGlobal().getRandomFloat();
	pickupRightCol.y = 0.66f;
	pickupRightCol.z = 0.75f;
	pickupRightCol = ColourHelpers::hslToRgb(pickupRightCol);

	tempCol.x = Random::getGlobal().getRandomFloat();
	tempCol.y = 0.66f;
	tempCol.z = 0.75f;
	tempCol = ColourHelpers::hslToRgb(tempCol);
	pickupLeftTween.reset(tempCol);
	pickupRightTween.reset(pickupLeftCol);

	for(int i=0;i<20;++i)
	{
		for(int j=0;j<11;++j)
			pickups.push_back(Pickup(TwoInts(i, j)));
	}

	pickupActions.push_back(L"PickupCircle");
	pickupActions.push_back(L"PickupCross1");
	pickupActions.push_back(L"PickupCross2");
	pickupActions.push_back(L"PickupDiamond");
	pickupActions.push_back(L"PickupDots1");
	pickupActions.push_back(L"PickupDots2");
	pickupActions.push_back(L"PickupEye");
	pickupActions.push_back(L"PickupHexagon");
	pickupActions.push_back(L"PickupLines1");
	pickupActions.push_back(L"PickupLines2");
	pickupActions.push_back(L"PickupPentagon");
	pickupActions.push_back(L"PickupSquare");
	pickupActions.push_back(L"PickupStar1");
	pickupActions.push_back(L"PickupStar2");
	pickupActions.push_back(L"PickupTriangle");
	pickupActions.push_back(L"PickupWave");

	sounder->registerCallback(this);
}

//------------------------------------------------------------------------------
GamePlayState::~GamePlayState()
{

}

//------------------------------------------------------------------------------
void GamePlayState::update(float delta)
{
	int i, j;
	ThreeFloats tempCol;
	std::wstring pickupType;

	for(i=(playerBullets.size()-1);i>=0;--i)
	{
		playerBullets[i]->update(delta);

		if(playerBullets[i]->getPosition().y > 752.0f)
			playerBullets.erase(playerBullets.begin()+i);
	}

	if(bulletCounter > 0.0f)
		bulletCounter -= delta;
	else if(fireIsDown)
	{
		bulletCounter = 30.0f;
		playerBullets.push_back(shared_ptr<PlayerBullet>(new PlayerBullet(player.getHeadPosition())));
	}

	pickupCollision();

	if(fillPos < TwoInts(19, 10))
	{
		bool already = false;

		for(i=0;i<pickups.size();++i)
		{
			if(pickups[i].getPosition() == fillPos)
			{
				already = true;
				break;
			}
		}

		if(!already)
			pickups.push_back(Pickup(fillPos));
		++fillPos.x;
		if((fillPos.x > 19) && (fillPos.y < 10))
		{
			fillPos.x = 0;
			++fillPos.y;
		}
	}

	pickupCount -= delta;
	if(pickupCount <= 0.0f)
	{
		std::vector<TwoInts> availableCells;

		pickupCount = 10.0f + (Random::getGlobal().getRandomFloat() * 50.0f);

		for(i=0;i<20;++i)
		{
			for(j=0;j<11;++j)
				availableCells.push_back(TwoInts(i, j));
		}

		//Make a vector of all the cells free of pickups and players/ghosts.
		for(i=0;i<pickups.size();++i)
		{
			for(j=0;j<availableCells.size();++j)
			{
				if(availableCells[j] == pickups[i].getPosition())
				{
					availableCells.erase(availableCells.begin()+j);
					break;
				}
			}
		}
		for(i=0;i<player.getNumSegments();++i)
		{
			for(j=0;j<availableCells.size();++j)
			{
				if(availableCells[j] == player.getSegmentPosition(i))
				{
					availableCells.erase(availableCells.begin()+j);
					break;
				}
			}
		}
		for(i=0;i<playerGhost.getNumSegments();++i)
		{
			for(j=0;j<availableCells.size();++j)
			{
				if(availableCells[j] == playerGhost.getSegmentPosition(i))
				{
					availableCells.erase(availableCells.begin()+j);
					break;
				}
			}
		}
		for(i=0;i<playerGhost2.getNumSegments();++i)
		{
			for(j=0;j<availableCells.size();++j)
			{
				if(availableCells[j] == playerGhost2.getSegmentPosition(i))
				{
					availableCells.erase(availableCells.begin()+j);
					break;
				}
			}
		}

		i = Random::getGlobal().getRandomInt(availableCells.size());
		if(availableCells.size())
		{
			if(nextPickup != L"")
			{
				pickupType = nextPickup;
				nextPickup = L"";
			}
			else if(Random::getGlobal().getRandomFloat() > 0.8f)
			{
				j = Random::getGlobal().getRandomInt(pickupActions.size());
				pickupType = pickupActions[j];
			}

			pickups.push_back(Pickup(availableCells[i], pickupType));
		}
	}

	//Update the sounder with the relevant respective amplitudes.
	{
		float tempf;
		int max = 0;

		if(globalData->getGameCounter().getCount(L"PlayerPickup") > max)
			max = globalData->getGameCounter().getCount(L"PlayerPickup");
		if(globalData->getGameCounter().getCount(L"Ghost1Pickup") > max)
			max = globalData->getGameCounter().getCount(L"Ghost1Pickup");
		if(globalData->getGameCounter().getCount(L"Ghost2Pickup") > max)
			max = globalData->getGameCounter().getCount(L"Ghost2Pickup");

		sounder->setNoteAmp((globalData->getGameCounter().getCount(L"PlayerPickup")/(float)max),
							(globalData->getGameCounter().getCount(L"Ghost1Pickup")/(float)max),
							(globalData->getGameCounter().getCount(L"Ghost2Pickup")/(float)max));
	}

	if(!backgroundTween.getFinished())
		backgroundTween.tick(delta);

	if(!pickupLeftTween.getFinished())
		pickupLeftTween.tick(delta);
	else
	{
		tempCol.x = Random::getGlobal().getRandomFloat();
		tempCol.y = 0.66f;
		tempCol.z = 0.75f;
		tempCol = ColourHelpers::hslToRgb(tempCol);
		pickupLeftTween.reset(tempCol);
		pickupRightTween.reset(pickupLeftCol);
	}
	if(!pickupRightTween.getFinished())
		pickupRightTween.tick(delta);

	particles.update();
}

//------------------------------------------------------------------------------
void GamePlayState::draw(DrawerPtr& d)
{
	int i;
	float tempf;
	ThreeFloats tempCol;

	d->fillRect(0.0f, d->getDrawingAreaSize(), background);

	for(i=0;i<pickups.size();++i)
	{
		tempf = (float)pickups[i].getPosition().x/20.0f;
		tempCol = pickupLeftCol * (1.0f-tempf);
		tempCol += pickupRightCol * tempf;
		pickups[i].draw(d, tempCol);
	}
	
	playerGhost2.draw(d);
	playerGhost.draw(d);
	player.draw(d);

	//particles.draw(d);

	for(i=0;i<(int)playerBullets.size();++i)
		playerBullets[i]->draw(d);
}

//------------------------------------------------------------------------------
void GamePlayState::keyDown(const std::wstring& action,
							const std::wstring& keyPressed,
							int keyModifier)
{
	if(action == L"Escape")
		quit();
	else if(action == L"P1Left")
	{
		directionKey = 0;
		player.move(1);
		updateSounderNotes();
	}
	else if(action == L"P1Up")
	{
		directionKey = 1;
		player.move(2);
		updateSounderNotes();
	}
	else if(action == L"P1Right")
	{
		directionKey = 2;
		player.move(3);
		updateSounderNotes();
	}
	else if(action == L"P1Down")
	{
		directionKey = 3;
		player.move(4);
		updateSounderNotes();
	}
	else if(action == L"Action")
		fireIsDown = true;
	else if(keyPressed == L"1")
		ghostLoop1 = !ghostLoop1;
	else if(keyPressed == L"2")
		ghostLoop2 = !ghostLoop2;
}

//------------------------------------------------------------------------------
void GamePlayState::keyUp(const std::wstring& action,
						  const std::wstring& keyPressed,
						  int keyModifier)
{
	if(action == L"Action")
		fireIsDown = false;
	else if((action == L"P1Left") && (directionKey == 0))
		directionKey = -1;
	else if((action == L"P1Up") && (directionKey == 1))
		directionKey = -1;
	else if((action == L"P1Right") && (directionKey == 2))
		directionKey = -1;
	else if((action == L"P1Down") && (directionKey == 3))
		directionKey = -1;
}

//------------------------------------------------------------------------------
void GamePlayState::mouseMove(const TwoFloats& pos)
{
	
}

//------------------------------------------------------------------------------
void GamePlayState::nextStepCallback(int step, int sample)
{
	int i;
	TwoInts tempPos;

	/*if(directionKey > -1)
	{
		switch(directionKey)
		{
			case 0: //Left.
				player.move(1);
				break;
			case 1: //Up.
				player.move(2);
				break;
			case 2: //Right.
				player.move(3);
				break;
			case 3: //Down.
				player.move(4);
				break;
		}
		updateSounderNotes();
	}*/

	if(!(step%2))
	{
		player.scaleSegment(step/2);
		playerGhost2.scaleSegment(step/2);
	}
	else
		playerGhost.scaleSegment(step/2);

	if(playerPerc > -1)
	{
		switch(playerPerc)
		{
			case 0:
				sounder->triggerKick(sample);
				break;
			case 1:
				sounder->triggerClap(sample);
				break;
			case 2:
				sounder->triggerHat(sample);
				break;
			case 3:
				sounder->triggerCrash(sample);
				break;
		}

		playerPerc = -1;
	}

	if(ghostLoop1)
	{
		playerGhost.loopOneSegment();
		updateSounderNotesHi();
	}
	if(ghostLoop2)
	{
		playerGhost2.loopOneSegment();
		updateSounderNotesOther();
	}

	if(!step)
	{
		ThreeFloats tempCol;
		const std::vector<Player::TwoIntThing>& segments = player.getSegments();
		const std::vector<PlayerGhost::TwoIntThing>& segmentsGhost = playerGhost.getSegments();
		
		playerGhost2.setColour(playerGhost.getColour() * 0.8f);
		playerGhost.setColour(player.getColour() * 0.8f);
		player.setColour(background * 0.8f);

		tempCol.x = Random::getGlobal().getRandomFloat();
		tempCol.y = 0.66f;
		tempCol.z = 0.75f;
		tempCol = ColourHelpers::hslToRgb(tempCol);

		backgroundTween.reset(tempCol);

		if(!ghostLoop2)
			sounder->updateNotesOther();
		if(!ghostLoop1)
			sounder->updateNotesHi();
		for(i=0;i<8;++i)
		{
			if(!ghostLoop2)
				playerGhost2.setSegment(i, segmentsGhost[i].pos.x, segmentsGhost[i].pos.y);
			if(!ghostLoop1)
				playerGhost.setSegment(i, segments[i].pos.x, segments[i].pos.y);
		}
	}

	//Handle percussion.
	for(i=(pickups.size()-1);i>=0;--i)
	{
		tempPos = pickups[i].getPosition();
		if(tempPos == playerGhost.getSegmentPosition(0))
		{
			/*pickups.erase(pickups.begin()+i);
			globalData->getGameCounter().incrementCount(L"Ghost1Pickup");*/
			switch(tempPos.x % 4)
			{
				case 0:
					sounder->triggerKick(sample);
					break;
				case 1:
					sounder->triggerClap(sample);
					break;
				case 2:
					sounder->triggerHat(sample);
					break;
				case 3:
					sounder->triggerCrash(sample);
					break;
			}
		}
		else if(tempPos == playerGhost2.getSegmentPosition(0))
		{
			/*pickups.erase(pickups.begin()+i);
			globalData->getGameCounter().incrementCount(L"Ghost2Pickup");*/
			switch(tempPos.x % 4)
			{
				case 0:
					sounder->triggerKick(sample);
					break;
				case 1:
					sounder->triggerClap(sample);
					break;
				case 2:
					sounder->triggerHat(sample);
					break;
				case 3:
					sounder->triggerCrash(sample);
					break;
			}
		}
	}
}

//------------------------------------------------------------------------------
void GamePlayState::updateSounderNotes()
{
	int i;
	int notes[8];
	const std::vector<Player::TwoIntThing>& positions = player.getSegments();

	if(playerGhost2.getSegmentPosition(0) != TwoInts(-1, -1))
	{
		if(ghostLoop1Count < 16)
		{
			++ghostLoop1Count;
			if(ghostLoop1Count > 15)
			{
				ghostLoop1 = true;
				ghostLoop2 = true;
			}
		}
	}

	notes[0] = 0;
	for(i=1;i<8;++i)
	{
		notes[i] = notes[i-1];
		if(positions[i].pos.x < positions[i-1].pos.x)
			--notes[i];
		else if(positions[i].pos.x > positions[i-1].pos.x)
			++notes[i];
		if(positions[i].pos.y < positions[i-1].pos.y)
			notes[i] -= 3;
		else if(positions[i].pos.y > positions[i-1].pos.y)
			notes[i] += 2;
		if(notes[i] < 0)
			notes[i] += 8;
		else if(notes[i] > 7)
			notes[i] -= 8;
	}

	for(i=(pickups.size()-1);i>=0;--i)
	{
		if(pickups[i].getPosition() == player.getSegmentPosition(0))
			playerPerc = pickups[i].getPosition().x % 4;
	}

	sounder->updateNotes(notes);
}

//------------------------------------------------------------------------------
void GamePlayState::updateSounderNotesHi()
{
	int i;
	int notes[8];
	const std::vector<PlayerGhost::TwoIntThing>& positions = playerGhost.getSegments();

	notes[0] = 0;
	for(i=1;i<8;++i)
	{
		notes[i] = notes[i-1];
		if(positions[i].pos.x < positions[i-1].pos.x)
			--notes[i];
		else if(positions[i].pos.x > positions[i-1].pos.x)
			++notes[i];
		if(positions[i].pos.y < positions[i-1].pos.y)
			notes[i] -= 3;
		else if(positions[i].pos.y > positions[i-1].pos.y)
			notes[i] += 2;
		if(notes[i] < 0)
			notes[i] += 8;
		else if(notes[i] > 7)
			notes[i] -= 8;
	}

	sounder->updateNotesHi(notes);
}

//------------------------------------------------------------------------------
void GamePlayState::updateSounderNotesOther()
{
	int i;
	int notes[8];
	const std::vector<PlayerGhost::TwoIntThing>& positions = playerGhost2.getSegments();

	notes[0] = 0;
	for(i=1;i<8;++i)
	{
		notes[i] = notes[i-1];
		if(positions[i].pos.x < positions[i-1].pos.x)
			--notes[i];
		else if(positions[i].pos.x > positions[i-1].pos.x)
			++notes[i];
		if(positions[i].pos.y < positions[i-1].pos.y)
			notes[i] -= 3;
		else if(positions[i].pos.y > positions[i-1].pos.y)
			notes[i] += 2;
		if(notes[i] < 0)
			notes[i] += 8;
		else if(notes[i] > 7)
			notes[i] -= 8;
	}

	sounder->updateNotesOther(notes);
}

//------------------------------------------------------------------------------
void GamePlayState::pickupCollision()
{
	int i;
	TwoInts tempPos;

	for(i=(pickups.size()-1);i>=0;--i)
	{
		tempPos = pickups[i].getPosition();
		if(tempPos == player.getSegmentPosition(0))
		{
			pickupAction(pickups[i].getType());

			//particles.trigger(TwoFloats(32.0f+((float)tempPos.x * 64.0f), 32.0f+((float)tempPos.y * 64.0f)));
			pickups.erase(pickups.begin()+i);
			globalData->getGameCounter().incrementCount(L"PlayerPickup");
			/*if(!ghostLoop1)
				ghostLoop1 = true;
			else if(!ghostLoop2)
				ghostLoop2 = true;*/
		}
		else if(tempPos == playerGhost.getSegmentPosition(0))
		{
			pickupAction(pickups[i].getType());

			pickups.erase(pickups.begin()+i);
			globalData->getGameCounter().incrementCount(L"Ghost1Pickup");
		}
		else if(tempPos == playerGhost2.getSegmentPosition(0))
		{
			pickupAction(pickups[i].getType());

			pickups.erase(pickups.begin()+i);
			globalData->getGameCounter().incrementCount(L"Ghost2Pickup");
		}
	}
}

//------------------------------------------------------------------------------
void GamePlayState::pickupAction(const std::wstring& pickupType)
{
	int i;

	if(pickupType == L"PickupCross1")
	{
		fillPos = 0;
	}
	else if(pickupType == L"PickupDots1")
	{
		musicPlaying = !musicPlaying;
		if(!musicPlaying)
			nextPickup = pickupType;

		sounder->setMusicPlaying(musicPlaying);
	}
	else if(pickupType == L"PickupSquare")
	{
		sounder->selectRandomPitches();
	}
	else if(pickupType == L"PickupTriangle")
	{
		ghostLoop1 = false;
		ghostLoop2 = false;
		ghostLoop1Count = 0;
		ghostLoop2Count = 0;
		playerGhost.resetSegments();
		playerGhost2.resetSegments();
		sounder->resetGhosts();
	}
	else if(pickupType == L"PickupStar2")
	{
		distortion = !distortion;
		if(distortion)
			nextPickup = L"PickupStar2";

		sounder->setDistortion(distortion);
	}
	else if(pickupType == L"PickupCircle")
	{
		int i;

		for(i=0;i<pickups.size();++i)
		{
			if(pickups[i].getType() == L"")
				pickups[i].setType(pickupActions[Random::getGlobal().getRandomInt(pickupActions.size())]);
			else
				pickups[i].setType(L"");

			while(pickups[i].getType() == L"PickupCircle")
				pickups[i].setType(pickupActions[Random::getGlobal().getRandomInt(pickupActions.size())]);
		}
	}
}
