//	GamePlayState.h - The main gameplay state.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef GAMEPLAYSTATE_H_
#define GAMEPLAYSTATE_H_

#include "ParticleGenerator.h"
#include "PastarenaSounder.h"
#include "PlayerBullet.h"
#include "PlayerGhost.h"
#include "GameState.h"
#include "Tweener.h"
#include "Pickup.h"
#include "Player.h"

#include <vector>

class CaterpillarGlobal;

///	The main gameplay state.
class GamePlayState : public GameState,
					  public PastarenaSounder::Callback
{
  public:
	///	Constructor.
	GamePlayState(Application& app, SounderPtr& sound);
	///	Destructor.
	~GamePlayState();

	///	Called frequently to update the state.
	void update(float delta);
	///	Called to draw the state.
	void draw(DrawerPtr& d);

	///	Called when a key down event is received.
	void keyDown(const std::wstring& action,
				 const std::wstring& keyPressed,
				 int keyModifier);
	///	Called when a key up event is received.
	void keyUp(const std::wstring& action,
			   const std::wstring& keyPressed,
			   int keyModifier);

	///	Used to test connected line drawing.
	void mouseMove(const TwoFloats& pos);

	///	Called when a new step occurs.
	void nextStepCallback(int step, int sample);

	///	Returns the name of this state.
	const std::wstring getName() const {return L"GamePlayState";};
	///	Used to construct instances of this state.
	static GameStateCreator<GamePlayState> creator;
  private:
	///	Helper method. Updates the sounder with the new notes from the caterpillar.
	void updateSounderNotes();
	///	Helper method. Updates the sounder with the new notes from the caterpillar.
	void updateSounderNotesHi();
	///	Helper method. Updates the sounder with the new notes from the caterpillar.
	void updateSounderNotesOther();
	///	Helper method. Tests whether the player's grabbed a pickup.
	void pickupCollision();
	///	Helper method. Handles special pickup actions.
	void pickupAction(const std::wstring& pickupType);

	///	The background colour.
	ThreeFloats background;
	///	The background tweener.
	STweener<ThreeFloats> backgroundTween;

	///	The particle generator.
	ParticleGenerator particles;

	///	The player caterpillar.
	Player player;
	///	The ghost caterpillar.
	PlayerGhost playerGhost;
	///	The 2nd ghost caterpillar.
	PlayerGhost playerGhost2;

	///	Used to trigger a percussion sound from the player collecting a pickup.
	int playerPerc;

	///	All the pickups currently on screen.
	std::vector<Pickup> pickups;
	///	Used to determine when to add new pickups.
	float pickupCount;
	///	The pickup left colour.
	ThreeFloats pickupLeftCol;
	///	For tweening the pickup left colour.
	STweener<ThreeFloats> pickupLeftTween;
	///	The pickup right colour.
	ThreeFloats pickupRightCol;
	///	For tweening the pickup right colour.
	STweener<ThreeFloats> pickupRightTween;

	///	True if the first ghost is looping.
	bool ghostLoop1;
	///	True if the second ghost is looping.
	bool ghostLoop2;
	///	Used to determine when to turn on ghostLoop1.
	int ghostLoop1Count;
	///	Used to determine when to turn on ghostLoop1.
	int ghostLoop2Count;
	///	Used to determine when ghost loops can be turned on.
	bool noLoops;

	///	The currently active player bullets.
	std::vector<shared_ptr<PlayerBullet> > playerBullets;
	///	Counter used to ensure the bullets don't fire too fast.
	float bulletCounter;
	///	True if the fire button's held down.
	bool fireIsDown;
	///	The direction key the player's holding (-1=none;0=left;1=up;2=right;3=down).
	int directionKey;

	///	The available pickup action types.
	std::vector<std::wstring> pickupActions;

	///	Used to fill the screen with pickups, one at a time.
	TwoInts fillPos;
	///	Used to stop/start the (non-perc) synths.
	bool musicPlaying;
	///	Used to ensure the next pickup which is added is a Dots (so we can turn the music back on).
	std::wstring nextPickup;
	///	Used to turn distortion on and off.
	bool distortion;

	///	Our global data.
	shared_ptr<CaterpillarGlobal> globalData;
	///	Our sounder.
	shared_ptr<PastarenaSounder> sounder;
};

#endif
