//	PastarenaSounder.cpp - The game's sounder.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "PastarenaSounder.h"
#include "Random.h"

#include "PastaPickup.inl"
#include "PastaThing.inl"
#include "PastaSlow.inl"
#include "PastaMine.inl"
#include "PastaPew.inl"
#include "PastaLo.inl"
#include "PastaHi.inl"

#include <cassert>
#include <sstream>
#include <string>

//	----------------------------------------------------------------------------
PastarenaSounder::PastarenaSounder():
musicPlaying(true),
distortion(false),
callback(0)
{
	int i;

	amplitudes[0] = 1.0f;
	amplitudes[1] = 1.0f;
	amplitudes[2] = 1.0f;

	hiSynth.loadParameters(PastaHi::numParameters, PastaHi::parameters);
	hiDelay.setDelay(0.125f);
	hiDelay.setFeedback(0.75f);

	loSynth.loadParameters(PastaLo::numParameters, PastaLo::parameters);
	loDelay.setDelay(0.25f);
	loDelay.setFeedback(0.75f);

	slowSynth.loadParameters(PastaSlow::numParameters, PastaSlow::parameters);
	slowDelay.setDelay(0.33f);
	slowDelay.setFeedback(0.75f);

	mineSynth.loadParameters(PastaMine::numParameters, PastaMine::parameters);
	mineDelay.setDelay(0.125f);
	mineDelay.setFeedback(0.5f);

	pickupSynth.loadParameters(PastaThing::numParameters, PastaThing::parameters);
	pickupDelay.setDelay(0.125f);
	pickupDelay.setFeedback(0.5f);

	pewSynth.loadParameters(PastaPew::numParameters, PastaPew::parameters);
	pewDelay.setDelay(0.125f);
	pewDelay.setFeedback(0.75f);

	percDelay.setDelay(0.125f);
	percDelay.setFeedback(0.5f);

	loadPitches();

	for(i=0;i<8;++i)
	{
		notes[i] = i;
		notesHi[i] = -1;
		notesOther[i] = -1;
	}
	cursor.registerCallback(this);
}

//	----------------------------------------------------------------------------
PastarenaSounder::~PastarenaSounder()
{

}

//	----------------------------------------------------------------------------
void PastarenaSounder::getAudio(float *input, float *output, int numSamples)
{
	int i;

	assert(numSamples <= (BufferSize/2));

	cursor.audioBlock(numSamples);

	for(i=0;i<(numSamples*2);++i)
	{
		loBuffer[i] = 0.0f;
		slowBuffer[i] = 0.0f;
		mineBuffer[i] = 0.0f;
		pickupBuffer[i] = 0.0f;
		pewBuffer[i] = 0.0f;
		percBuffer[i] = 0.0f;

		output[i] = 0.0f;
	}

	hiSynth.getAudio(output, numSamples);
	hiDelay.getAudio(output, numSamples);

	loSynth.getAudio(loBuffer, numSamples);
	loDelay.getAudio(loBuffer, numSamples);

	slowSynth.getAudio(slowBuffer, numSamples);
	slowDelay.getAudio(slowBuffer, numSamples);

	mineSynth.getAudio(mineBuffer, numSamples);
	mineDelay.getAudio(mineBuffer, numSamples);

	pickupSynth.getAudio(pickupBuffer, numSamples);
	pickupDelay.getAudio(pickupBuffer, numSamples);

	pewSynth.getAudio(pewBuffer, numSamples);
	pewDelay.getAudio(pewBuffer, numSamples);

	kick.getAudio(percBuffer, numSamples);
	if(clap.getActive())
		clap.getAudio(percBuffer, numSamples);
	if(hat.getActive())
		hat.getAudio(percBuffer, numSamples);
	if(crash.getActive())
		crash.getAudio(percBuffer, numSamples);
	percDelay.getAudio(percBuffer, numSamples);

	for(i=0;i<(numSamples*2);++i)
	{
		output[i] += loBuffer[i];
		output[i] += slowBuffer[i];
		output[i] += mineBuffer[i];
		output[i] += pickupBuffer[i];
		output[i] += pewBuffer[i];
	}

	if(distortion)
	{
		for(i=0;i<(numSamples*2);++i)
			output[i] = tanhf(output[i] * 24.0f) * 0.2f;
	}

	for(i=0;i<(numSamples*2);++i)
		output[i] += percBuffer[i] * 0.5f;
}

//	----------------------------------------------------------------------------
void PastarenaSounder::triggerHi(float val)
{
	int index = (int)(val * 7.99f);

	hiSynth.triggerNote(pitches[index][currentPitches]*2.0f, 0);
}

//	----------------------------------------------------------------------------
void PastarenaSounder::triggerLo(float val)
{
	int index = (int)(val * 7.99f);

	loSynth.triggerNote(pitches[index][currentPitches], 0);
}

//	----------------------------------------------------------------------------
void PastarenaSounder::triggerSlow(float val)
{
	int index = (int)(val * 7.99f);

	slowSynth.triggerNote(pitches[index][currentPitches], 0);
}

//	----------------------------------------------------------------------------
void PastarenaSounder::triggerMine(float val)
{
	int index = (int)(val * 7.99f);

	mineSynth.triggerNote(pitches[index][currentPitches], 0);
}

//	----------------------------------------------------------------------------
void PastarenaSounder::triggerPickup(float val)
{
	int index = (int)(val * 7.99f);

	pickupSynth.triggerNote(pitches[index][currentPitches], 0);
}

//	----------------------------------------------------------------------------
void PastarenaSounder::triggerPew(float val)
{
	int index = (int)(val * 7.99f);

	pewSynth.triggerNote(pitches[index][currentPitches], 0);
}

//	----------------------------------------------------------------------------
void PastarenaSounder::triggerKick(int sample)
{
	kick.triggerNote(440.0f, sample, 1.0f);
}

//	----------------------------------------------------------------------------
void PastarenaSounder::triggerClap(int sample)
{
	clap.setActive(true, sample, false);
}

//	----------------------------------------------------------------------------
void PastarenaSounder::triggerHat(int sample)
{
	hat.setActive(true, sample, false);
}

//	----------------------------------------------------------------------------
void PastarenaSounder::triggerCrash(int sample)
{
	crash.setActive(true, sample, false);
}

//	----------------------------------------------------------------------------
void PastarenaSounder::updateNotes(int *val)
{
	int i;

	for(i=0;i<8;++i)
		notes[i] = val[i];
}

//	----------------------------------------------------------------------------
void PastarenaSounder::updateNotesHi(int *val)
{
	int i;

	if(val)
	{
		for(i=0;i<8;++i)
			notesHi[i] = val[i];
	}
	else
	{
		for(i=0;i<8;++i)
			notesHi[i] = notes[i];
	}
}

//	----------------------------------------------------------------------------
void PastarenaSounder::updateNotesOther(int *val)
{
	int i;

	if(val)
	{
		for(i=0;i<8;++i)
			notesOther[i] = val[i];
	}
	else
	{
		for(i=0;i<8;++i)
			notesOther[i] = notes[i];
	}
}

//	----------------------------------------------------------------------------
void PastarenaSounder::setMusicPlaying(bool val)
{
	musicPlaying = val;
}

//	----------------------------------------------------------------------------
void PastarenaSounder::resetGhosts()
{
	int i;

	for(i=0;i<8;++i)
	{
		notesHi[i] = -1;
		notesOther[i] = -1;
	}
}

//	----------------------------------------------------------------------------
void PastarenaSounder::setDistortion(bool val)
{
	distortion = val;
}

//	----------------------------------------------------------------------------
void PastarenaSounder::setNoteAmp(float player, float ghost1, float ghost2)
{
	amplitudes[0] = player;
	amplitudes[1] = ghost1;
	amplitudes[2] = ghost2;
}

//	----------------------------------------------------------------------------
void PastarenaSounder::nextStepCallback(int step, int sample)
{
	int tempint;

	if(!(step%2) && musicPlaying)
	{
		loSynth.triggerNote(pitches[notes[step/2]][currentPitches], sample, amplitudes[0]);
		tempint = notesOther[step/2];
		if(tempint > -1)
			pickupSynth.triggerNote(pitches[notesOther[step/2]][currentPitches], sample, amplitudes[2]);
	}
	else if(musicPlaying)
	{
		tempint = notesHi[step/2];
		if(tempint > -1)
			hiSynth.triggerNote(pitches[notesHi[step/2]][currentPitches], sample, amplitudes[1]);
	}

	if(callback)
		callback->nextStepCallback(step, sample);
}

//	----------------------------------------------------------------------------
void PastarenaSounder::loadPitches()
{
	int i;
	int index;
	FilePath pitchesDir = FilePath::getDataFilesDir().getChild(L"pitches");

	for(i=0;i<pitchesDir.getNumChildren();++i)
	{
		FilePath pitchFile = pitchesDir.getChild(i);

		if(pitchFile.exists())
		{
			if(pitchFile.getExtension() == L"txt")
			{
				UTF8File phil(pitchFile);

				if(!phil.getFail())
				{
					int i;
					float tempf;
					std::wstring tempstr;
					std::wstringstream tempstr2;

					i = 0;
					while(!phil.readLine(tempstr))
					{
						assert(i < 8);

						tempstr2.clear();
						tempstr2.str(L"");
						tempstr2 << tempstr;

						tempstr2 >> tempf;

						pitches[i].push_back(tempf * 0.25f);

						++i;
					}
				}
			}
		}
	}
	currentPitches = Random::getGlobal().getRandomInt(pitches[0].size());
}

//	----------------------------------------------------------------------------
void PastarenaSounder::selectRandomPitches()
{
	int index = Random::getGlobal().getRandomInt(pitches[0].size());

	while(index == currentPitches)
		index = Random::getGlobal().getRandomInt(pitches[0].size());

	currentPitches = index;
}

//	----------------------------------------------------------------------------
void PastarenaSounder::registerCallback(Callback *c)
{
	callback = c;
}
