//	PastarenaSounder.h - The game's sounder.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef PASTARENASOUNDER_H_
#define PASTARENASOUNDER_H_

#include "SequencerCursor.h"

#include "synth/PolyphonicSynth.h"
#include "synth/SimpleSynth.h"
#include "synth/CrashSynth.h"
#include "synth/KickSynth2.h"
#include "synth/ClapSynth.h"
#include "synth/HatSynth.h"
#include "sound/Sounder.h"
#include "sound/Delay.h"

///	The game's sounder.
class PastarenaSounder : public Sounder,
						 public SequencerCursor::Callback
{
  public:
	///	Constructor.
	PastarenaSounder();
	///	Destructor.
	~PastarenaSounder();

	///	Called for every audio block.
	void getAudio(float *input, float *output, int numSamples);

	///	Called for each sound file resource by LoadingScreen.
	void loadSoundFileResource(SoundFileLoaderPtr soundFileLoader) {};

	///	Triggers a synth note.
	/*!
		\param val A value 0->1 which will be mapped to a particular pitch.
	 */
	void triggerHi(float val);
	///	Triggers a synth note.
	/*!
		\param val A value 0->1 which will be mapped to a particular pitch.
	 */
	void triggerLo(float val);
	///	Triggers a synth note.
	/*!
		\param val A value 0->1 which will be mapped to a particular pitch.
	 */
	void triggerSlow(float val);
	///	Triggers a synth note.
	/*!
		\param val A value 0->1 which will be mapped to a particular pitch.
	 */
	void triggerMine(float val);
	///	Triggers a synth note.
	/*!
		\param val A value 0->1 which will be mapped to a particular pitch.
	 */
	void triggerPickup(float val);
	///	Triggers a synth note.
	/*!
		\param val A value 0->1 which will be mapped to a particular pitch.
	 */
	void triggerPew(float val);

	///	Triggers a kick.
	void triggerKick(int sample);
	///	Triggers a clap.
	void triggerClap(int sample);
	///	Triggers a hat.
	void triggerHat(int sample);
	///	Triggers a crash.
	void triggerCrash(int sample);

	///	Updates the caterpillar notes.
	void updateNotes(int *val);
	///	Updates the hi notes.
	void updateNotesHi(int *val = 0);
	///	Updates the other notes.
	void updateNotesOther(int *val = 0);

	///	Starts/stops the (non-perc) synths playing.
	void setMusicPlaying(bool val);
	///	Resets the ghosts.
	void resetGhosts();
	///	Turns the synth distortion on/off.
	void setDistortion(bool val);

	///	Sets the individual instruments' amplitudes.
	void setNoteAmp(float player, float ghost1, float ghost2);

	///	Called whenever the cursor moves to the next step in the sequence.
	void nextStepCallback(int step, int sample);

	///	Loads all the available pitches.
	void loadPitches();
	///	Selects a random pitch set (different to the current one).
	void selectRandomPitches();

	///	Callback class used to tell GamePlayState when a note should be emphasized.
	class Callback
	{
	  public:
		///	Called when a new step occurs.
		virtual void nextStepCallback(int step, int sample) = 0;
	};
	///	Registers a callback to call for each step.
	void registerCallback(Callback *c);
  private:
	///	Some constants.
	enum
	{
		BufferSize = 16384
	};

	///	The hi synth.
	PolyphonicSynth<SimpleSynth> hiSynth;
	///	The hi synth delay.
	Delay hiDelay;

	///	The lo synth.
	PolyphonicSynth<SimpleSynth> loSynth;
	///	The lo synth delay.
	Delay loDelay;
	///	Temporary buffer for the lo synth.
	float loBuffer[BufferSize];

	///	The slow synth
	PolyphonicSynth<SimpleSynth> slowSynth;
	///	The slow synth delay.
	Delay slowDelay;
	///	Temporary buffer for the slow synth.
	float slowBuffer[BufferSize];

	///	The mine synth
	PolyphonicSynth<SimpleSynth> mineSynth;
	///	The slow mine delay.
	Delay mineDelay;
	///	Temporary buffer for the mine synth.
	float mineBuffer[BufferSize];

	///	The pickup synth
	PolyphonicSynth<SimpleSynth> pickupSynth;
	///	The slow mine delay.
	Delay pickupDelay;
	///	Temporary buffer for the mine synth.
	float pickupBuffer[BufferSize];

	///	The pew synth
	PolyphonicSynth<SimpleSynth> pewSynth;
	///	The pew delay.
	Delay pewDelay;
	///	Temporary buffer for the pew synth.
	float pewBuffer[BufferSize];

	///	The Kick synth.
	PolyphonicSynth<KickSynth2> kick;
	///	The Clap synth.
	ClapSynth clap;
	///	The Hat synth.
	HatSynth hat;
	///	The Crash synth.
	CrashSynth crash;
	///	The pew delay.
	Delay percDelay;
	///	Temporary buffer for the pew synth.
	float percBuffer[BufferSize];

	///	For triggering notes.
	SequencerCursor cursor;

	///	The individual instruments' amplitudes.
	float amplitudes[3];

	///	Pitch indices for the 8 notes.
	int notes[8];
	///	Pitch indices for the 8 hi notes.
	int notesHi[8];
	///	Pitch indices for the 8 other notes.
	int notesOther[8];

	///	Whether or not the music is currently playing.
	bool musicPlaying;
	///	True if the synths are currently being distorted.
	bool distortion;

	///	The current pitch set.
	int currentPitches;
	///	The available pitch sets
	std::vector<float> pitches[8];

	///	The callback to call for each step.
	Callback *callback;
};

#endif
