//	SequencerCursor.h - Class which keeps track of our position within the
//						16-note sequence.
//  ----------------------------------------------------------------------------
//	This file is part of 'GameTemplate', template code for a game.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef SEQUENCERCURSOR_H_
#define SEQUENCERCURSOR_H_

///	Class which keeps track of our position within the 16-note sequence.
class SequencerCursor
{
  public:
	///	Constructor.
	SequencerCursor();
	///	Destructor.
	~SequencerCursor();

	///	Called for every audio sample.
	/*!
		This will update our position within the sequence, and trigger a
		callback if we've moved on to the next step.
	 */
	void audioBlock(int numSamples);

	///	Resets our current position.
	void resetPosition();
	///	Sets the tempo.
	void setTempo(float val);

	///	Returns our current position within in the sequence.
	/*!
		\return 0->15.9999999...
	 */
	float getPosition() const {return position;};

	///	Simple callback class for anyone who wants to be informed when we move onto the next step.
	class Callback
	{
	  public:
		///	Destructor.
		virtual ~Callback() {};

		///	Called whenever the cursor moves to the next step in the sequence.
		/*!
			\param step The new step we've moved to.
			\param sample The sample within the current audio block at which we
			moved to the next step.

			This method will only ever be called from the audio thread.
		 */
		virtual void nextStepCallback(int step, int sample) = 0;
	};
	///	Registers a Callback to be called whenever the cursor moves to the next step in the sequence.
	void registerCallback(Callback *c);
  private:
	///	Our current position within the sequence.
	float position;
	///	Our current tempo.
	float tempo;

	///	A callback to call whenever the cursor moves to the next step in the sequence.
	Callback *callback;
};

#endif
